/**
  ******************************************************************************
  * @file    i2c.c
  * @author  Thomas R.
  * @version V1.0
  * @date    15/11/15
  * @brief   i2c Driver
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "nrf_soc.h"
#include "app_trace.h"
#include "i2c.h"

#if defined HELENA_DEBUG_RTT
#include "SEGGER_RTT.h"
#endif

/* External variables --------------------------------------------------------*/

/* Private typedef -----------------------------------------------------------*/

/* Private macros ------------------------------------------------------------*/

/* Private defines -----------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
const nrf_drv_twi_t twi_instance = NRF_DRV_TWI_INSTANCE(1);
const nrf_drv_twi_config_t twi_config =
{
    .scl                    = 29,
    .sda                    = 28,
    .frequency              = NRF_TWI_FREQ_250K,
    .interrupt_priority     = NRF_APP_PRIORITY_LOW
};

/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/
void i2c_Recover()
{
    NRF_TWI1->EVENTS_ERROR = 0;
    NRF_TWI1->ENABLE       = TWI_ENABLE_ENABLE_Disabled << TWI_ENABLE_ENABLE_Pos;
    NRF_TWI1->POWER        = 0;
    nrf_delay_us(5);
    NRF_TWI1->POWER        = 1;
    NRF_TWI1->ENABLE       = TWI_ENABLE_ENABLE_Enabled << TWI_ENABLE_ENABLE_Pos;
    app_trace_log("[I2C]: i2c recovered\r\n");
    (void)i2c_Init();
}

/* Public functions ----------------------------------------------------------*/
uint32_t i2c_Init()
{
    uint32_t err_code;
    err_code = nrf_drv_twi_init(&twi_instance, &twi_config, NULL);
    app_trace_log("[I2C]: i2c bus initialized\r\n");
    if (err_code == NRF_SUCCESS)
    {
        nrf_drv_twi_enable(&twi_instance);
        nrf_delay_ms(100);
    }
    return err_code;
}

void i2c_Deinit()
{
    nrf_drv_twi_uninit(&twi_instance);
}

uint32_t i2c_read(uint8_t device_address, uint8_t register_address, uint8_t length, uint8_t *data)
{
    uint32_t err_code;
    uint8_t retry_cnt = 2;                  /**< retry counter, in case of acknowledge error, communication is tryed again (MPU sometimes doesn't acknowledge) */

    do
    {                                       /**< send register address to read from */
        err_code = nrf_drv_twi_tx(&twi_instance, device_address, &register_address, 1, true);
        if (err_code != NRF_SUCCESS)        /**< if transmission failed, check why */
        {
            if (NRF_TWI1->ERRORSRC)         /**< in case of acknowledge error */
            {
                NRF_TWI1->ERRORSRC = 0;     /**< delete error source */
                if (--retry_cnt == 0)
                    return err_code;        /**< return error, if error occured twice */
                else
                    continue;               /**< otherwise try again */
            }
            else                            /**< in case of timeout error, PAN56 might be the reason, so recover bus and start over */
            {
                i2c_Recover();
                continue;
            }
        }
    }   while(0);

    retry_cnt = 2;
    do
    {                                       /**< read data */
        err_code = nrf_drv_twi_rx(&twi_instance, device_address, data, length, false);
        if (err_code != NRF_SUCCESS)        /**< if transmission failed, check why */
        {
            if (NRF_TWI1->ERRORSRC)         /**< in case of acknowledge error */
            {
                NRF_TWI1->ERRORSRC = 0;     /**< delete error source */
                if (--retry_cnt == 0)
                    return err_code;        /**< return error, if error occured twice */
                else
                    continue;               /**< otherwise try again */
            }
            else                            /**< in case of timeout error, PAN56 might be the reason, so recover bus and start over */
            {
                i2c_Recover();
                continue;
            }
        }
    }   while(0);

    return err_code;
}

uint32_t i2c_write(uint8_t device_address, uint8_t register_address, uint8_t length, uint8_t *data)
{
    uint8_t buffer[I2CBUFFERSIZE], retry_cnt = 2;
    uint32_t err_code;

    if (length > I2CBUFFERSIZE-1)
        return NRF_ERROR_INVALID_LENGTH;
    buffer[0] = register_address;
    memcpy(&buffer[1], data, length);

    do
    {                                       /**< send data */
        err_code = nrf_drv_twi_tx(&twi_instance, device_address, buffer, length+1, false);
        if (err_code != NRF_SUCCESS)        /**< if transmission failed, check why */
        {
            if (NRF_TWI1->ERRORSRC)         /**< in case of acknowledge error */
            {
                NRF_TWI1->ERRORSRC = 0;     /**< delete error source */
                if (--retry_cnt == 0)
                    return err_code;        /**< return error, if error occured twice */
                else
                    continue;               /**< otherwise try again */
            }
            else                            /**< in case of timeout error, PAN56 might be the reason, so recover bus and start over */
            {
                i2c_Recover();
                continue;
            }
        }
    }   while(0);

    return err_code;
}

/**END OF FILE*****************************************************************/



